/* --------------------------------------------------

   FAMICOM BASIC keyboard read driver.
   "kome wo kame"
   
   Programed by K.Kunikane. (rerofumi)
   
---------------------------------------------------- */

/* ----------------------- */
/* --- include             */
/* ----------------------- */
#include <m8c.h>
#include "fc2hid_table.h"

extern void StartSound(BYTE index);

/* ----------------------- */
/* --- global work         */
/* ----------------------- */

BYTE keyin_count;
BYTE keyin_ptr;
BYTE keyin_wait;
BYTE keyin_push;
BYTE keyin_trigger;

/* --- data buffer */
BYTE keyBuffer[16];

/* --- endpoint buffer */
BYTE report_ptr;
BYTE hidReport[8];


/* ----------------------- */
/* --- local method        */
/* ----------------------- */
BYTE HidUsage(BYTE ASCII)
{
// Convert the ASCII character into a HID usage
// This routine implements printable characters but not Shift
	if (ASCII == 0x0a) return 0x28;		// CR
	if ((ASCII > 0x7F) | (ASCII < 0x20)) return 0;
	return HidUsageTable[ASCII - 0x20];
}

void SetUsage(BYTE usg)
{
	if (report_ptr < 8) {
		hidReport[report_ptr] = usg;
		report_ptr++;
	}
}

/* ----------------------- */
/* --- interface           */
/* ----------------------- */

/* --- GPIO initialize */
void InitializeKey(void)
{
	PRT2DR = 0xff;
	keyin_count = 0;
	keyin_ptr = 0;
	keyin_wait = 0;
	keyin_push = 0;
	keyin_trigger = 0;
}

/* --- GPIO read work */
void ReadKey(void)
{
	BYTE inkey;

	if (keyin_count > 0) {
		return;
	}
	
	/* --- key */
	inkey = PRT2DR;
	if ((keyin_trigger == 1) && (inkey != 0xff)) {
		return;
	}

	keyin_trigger = 0;

	if ((inkey & 0b01000000) == 0) {
		keyBuffer[0] = 'U';
		keyBuffer[1] = 'P';
		keyBuffer[2] = 0x0a;
		keyBuffer[3] = 'O';
		keyBuffer[4] = 'T';
		keyBuffer[5] = 'U';
		keyBuffer[6] = ' ';
		keyBuffer[7] = 0x0a;
		keyBuffer[8] = 0x0a;
		keyin_count = 9;
		keyin_ptr = 0;
		keyin_trigger = 1;
		StartSound(0);
	}
	else if ((inkey & 0b10000000) == 0) {
		keyBuffer[0] = 'S';
		keyBuffer[1] = 'U';
		keyBuffer[2] = 'G';
		keyBuffer[3] = 'E';
		keyBuffer[4] = '-';
		keyBuffer[5] = 0x0a;
		keyBuffer[6] = 0x0a;
		keyin_count = 7;
		keyin_ptr = 0;
		keyin_trigger = 1;
		StartSound(1);
	}
	else if ((inkey & 0b00100000) == 0) {
		keyBuffer[0] = 'I';
		keyBuffer[1] = 'I';
		keyBuffer[2] = 'N';
		keyBuffer[3] = 'E';
		keyBuffer[4] = 0x0a;
		keyBuffer[5] = 0x0a;
		keyin_count = 6;
		keyin_ptr = 0;
		keyin_trigger = 1;
		StartSound(2);
	}
	else if ((inkey & 0b00001000) == 0) {
		keyBuffer[0] = 'G';
		keyBuffer[1] = 'J';
		keyBuffer[2] = 0x0a;
		keyBuffer[3] = 0x0a;
		keyin_count = 4;
		keyin_ptr = 0;
		keyin_trigger = 1;
		StartSound(3);
	}
	else if ((inkey & 0b00000010) == 0) {
		keyBuffer[0] = '8';
		keyBuffer[1] = '8';
		keyBuffer[2] = '8';
		keyBuffer[3] = '8';
		keyBuffer[4] = '8';
		keyBuffer[5] = 0x0a;
		keyBuffer[6] = 0x0a;
		keyin_count = 7;
		keyin_ptr = 0;
		keyin_trigger = 1;
		StartSound(4);
	}
}

/* --- build HID Usage Report */
BYTE SetHIDUsage(void)
{
	BYTE mod, usg;
	BYTE dosend;
	
	report_ptr = 2;
	mod = 0;
	dosend = 0;
	
	/* --- modifier key */
	hidReport[0] = mod;
	hidReport[1] = 0;
	hidReport[2] = 0;
	hidReport[3] = 0;
	hidReport[4] = 0;
	hidReport[5] = 0;
	hidReport[6] = 0;
	hidReport[7] = 0;
	
	//SetUsage(HidUsage('A'));
	keyin_wait += 1;
	if (keyin_wait > 10) { 
		if ((keyin_count > 0) && (keyin_push == 0)) {
			usg = keyBuffer[keyin_ptr];
			SetUsage(HidUsage(usg));
			keyin_ptr += 1;
			keyin_count -= 1;
			keyin_push = 1;
			dosend = 1;
		}
		else if (keyin_push == 1) {
			keyin_push = 0;
			dosend = 1;
		}
		keyin_wait = 0;
	}

	return dosend;
}

