//----------------------------------------------------------------------------
// C main line
//----------------------------------------------------------------------------

#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules

#include "fckey.h"
#include "tonetable.h"

extern BYTE SOF_Flag;

//o5c  60
//o5c# 61
//o5d  62
//o5d# 63
//o5e  64
//o5f  65
//o5f# 66
//o5g  67
//o5g# 68
//o5a  69
//o5a# 70
//o5b  71
//o6c  72
const BYTE ToneSeq[] = {
//1 (0)
65,2,
72,2,
69,4,
0,0,
//2 (4)
65,2,
69,6,
0,0,
//3 (7)
69,2,
68,2,
67,2,
0,0,
//4 (11)
67,2,
0,2,
71,2,
70,2,
0,0,
//5 (16)
71,2,
0,1,
71,1,
69,2,
0,1,
69,1,
71,1,
69,1,
71,1,
69,1,
0,0
};

/* ------------------------- */

BYTE bIsSound;
BYTE bSoundDataPtr;
BYTE bSoundTone;
BYTE bSoundCount;


/* ------------------------- */

void SoundSw(BYTE sw)
{
	BYTE s, rs;
	
	s = 0b00000001;
	rs = s ^ 0xff;
	
	if (sw) {
		PRT3GS |= s;
	}
	else {
		PRT3DR &= rs;
		PRT3GS &= rs;
	}
}

void SoundTone(BYTE index)
{
	PWM16_1_WritePeriod(ToneTable[index]);
	PWM16_1_WritePulseWidth(ToneTable[index]/4);
}

void StartSound(BYTE index)
{
	switch(index) {
		case 0:
			bSoundDataPtr = 0;
			break;
		case 1:
			bSoundDataPtr = 4;
			break;
		case 2:
			bSoundDataPtr = 7;
			break;
		case 3:
			bSoundDataPtr = 11;
			break;
		case 4:
			bSoundDataPtr = 16;
			break;
	}
	
	bSoundDataPtr += bSoundDataPtr;
	bSoundTone = ToneSeq[bSoundDataPtr];
	bSoundDataPtr += 1;
	bSoundCount = ToneSeq[bSoundDataPtr] * 4;
	bSoundDataPtr += 1;
	bIsSound = 1;
	SoundSw(1);
	SoundTone(bSoundTone);
}


/* ---------------------------------------- */
/* --- interrupt                            */
/* ---------------------------------------- */

#pragma interrupt_handler Timer1Int
void Timer1Int(void)
{
	if (bIsSound) {
		if (bSoundCount == 0) {
			bSoundTone = ToneSeq[bSoundDataPtr];
			bSoundDataPtr += 1;
			bSoundCount = ToneSeq[bSoundDataPtr] * 4;
			bSoundDataPtr += 1;
			if ((bSoundTone == 0) && (bSoundCount == 0)) {
				bIsSound = 0;
				SoundSw(0);
			}
			else {
				if (bSoundTone) {
					SoundSw(1);
					SoundTone(bSoundTone);
				}
				else {
					SoundSw(0);
				}					
			}
		}
		bSoundCount -= 1;
	}
}


/* ---------------------------------------- */
/* --- main loop                            */
/* ---------------------------------------- */
void main(void)
{
	InitializeKey();
	// Connect to the host
	USBFS_1_Start(0,USB_5V_OPERATION);
	PWM16_1_Start();
    Counter16_1_Start();
	Counter16_1_EnableInt();
	M8C_EnableGInt;
	
	SoundSw(0);
	
	// Wait to be enumerated
	while (!USBFS_1_bGetConfiguration());

	SOF_Flag = 0;
	USBFS_1_INT_REG |= USBFS_1_INT_SOF_MASK;
	
	while(1) {
		if (SOF_Flag) {
			//while(!USBFS_1_bGetEPState(2) != IN_BUFFER_EMPTY);
			ReadKey();
			if (SetHIDUsage()) {
				USBFS_1_LoadInEP(2, &hidReport[0], 8, USB_TOGGLE);
			}
			SOF_Flag = 0;
		}
	}
}
